@@ -36,9 +36,11 @@
 #include "pkg.h"
 #include "file_util.h"
 #include "str_util.h"
-#include "xregex.h"
 #include "libbb/libbb.h"
 
+#include <fnmatch.h>
+
+
 #include "ipkg_download.h"
 #include "ipkg_install.h"
 #include "ipkg_upgrade.h"
@@ -117,6 +119,8 @@ static void write_status_files_if_changed(ipkg_conf_t *conf)
 		       "  writing status file\n");
 	  ipkg_conf_write_status_files(conf);
 	  pkg_write_changed_filelists(conf);
+     } else { 
+	  ipkg_message(conf, IPKG_NOTICE, "Nothing to be done\n");
      }
 }
 
@@ -145,10 +149,9 @@ int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, const char **arg
 	p_userdata = userdata;
 	result = (cmd->fun)(conf, argc, argv);
         if ( result == 0 ) {
-           ipkg_message(conf, IPKG_NOTICE, "Successfully done.\n");
+           ipkg_message(conf, IPKG_NOTICE, "Successfully terminated.\n");
         } else {
-           ipkg_message(conf, IPKG_NOTICE, "Return value != 0.\n");
-           ipkg_message(conf, IPKG_NOTICE, "Please file a bug in bugzilla. Return code was %d.\n",result);
+           ipkg_message(conf, IPKG_NOTICE, "An error ocurred, return value: %d.\n", result);
         }
    
 	p_userdata = NULL;
@@ -310,7 +313,7 @@ int ipkg_multiple_files_scan(ipkg_conf_t *conf, int argc, char **argv)
      return 0;
 }
 
-int ipkg_configure_packages(ipkg_conf_t *conf, regex_t *re)
+int ipkg_configure_packages(ipkg_conf_t *conf, char *pkg_name)
 {
      pkg_vec_t *all;
      int i;
@@ -326,7 +329,7 @@ int ipkg_configure_packages(ipkg_conf_t *conf, regex_t *re)
      for(i = 0; i < all->len; i++) {
 	  pkg = all->pkgs[i];
 
-	  if (re && xregexec(re, pkg->name, 0, 0, 0)) 
+	  if (pkg_name && fnmatch(pkg_name, pkg->name, 0)) 
 	       continue;
 
 	  if (pkg->state_status == SS_UNPACKED) {
@@ -380,8 +383,9 @@ static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	  arg = argv[i];
 	  if (conf->multiple_providers)
 	       err = ipkg_install_multi_by_name(conf, arg);
-	  else
+	  else{
 	       err = ipkg_install_by_name(conf, arg);
+          }
 	  if (err == IPKG_PKG_HAS_NO_CANDIDATE) {
 	       ipkg_message(conf, IPKG_ERROR,
 			    "Cannot find package %s.\n"
@@ -397,7 +401,7 @@ static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
      write_status_files_if_changed(conf);
 
-     return 0;
+     return err;
 }
 
 static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char **argv)
@@ -437,8 +441,9 @@ static int ipkg_upgrade_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	       }
 	       if (pkg)
 		    ipkg_upgrade_pkg(conf, pkg);
-	       else 
+	       else {
 		    ipkg_install_by_name(conf, arg);
+               }
 	  }
      } else {
 	  pkg_vec_t *installed = pkg_vec_alloc();
@@ -497,39 +502,26 @@ static int ipkg_download_cmd(ipkg_conf_t *conf, int argc, char **argv)
      return 0;
 }
 
-static int ipkg_regcomp(regex_t *re, const char *pattern)
-{
-     char *delimited_pattern = NULL;
-     int err;
-     sprintf_alloc(&delimited_pattern, "^%s$", pattern);
-     err = xregcomp(re, delimited_pattern, REG_EXTENDED);
-     free(delimited_pattern);
-     return err;
-}
 
 static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
-     int i, err;
+     int i ;
      pkg_vec_t *available;
      pkg_t *pkg;
      char desc_short[IPKG_LIST_DESCRIPTION_LENGTH];
      char *newline;
-     regex_t re;
      char *pkg_name = NULL;
+     char *version_str;
 
      if (argc > 0) {
 	  pkg_name = argv[0];
-	  err = ipkg_regcomp(&re, pkg_name);
-	  if (err) {
-	       return err;
-	  }
      }
      available = pkg_vec_alloc();
      pkg_hash_fetch_available(&conf->pkg_hash, available);
      for (i=0; i < available->len; i++) {
 	  pkg = available->pkgs[i];
 	  /* if we have package name or pattern and pkg does not match, then skip it */
-	  if (pkg_name && xregexec(&re, pkg->name, 0, 0, 0)) 
+	  if (pkg_name && fnmatch(pkg_name, pkg->name, 0)) 
 	       continue;
 	  if (pkg->description) {
 	       strncpy(desc_short, pkg->description, IPKG_LIST_DESCRIPTION_LENGTH);
@@ -544,35 +536,32 @@ static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv)
 #ifndef IPKG_LIB
 	  printf("%s - %s\n", pkg->name, desc_short);
 #else
-	  if (ipkg_cb_list) ipkg_cb_list(pkg->name,desc_short,
-		                             pkg_version_str_alloc(pkg),
+	  if (ipkg_cb_list) {
+	  	version_str = pkg_version_str_alloc(pkg);
+	  	ipkg_cb_list(pkg->name,desc_short,
+		                             version_str,
 	                                 pkg->state_status,
 	                                 p_userdata);
+		free(version_str);
+	  }
 #endif
      }
 
-     if (pkg_name)
-	  xregfree(&re);
      return 0;
 }
 
 static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char **argv, int installed_only)
 {
-     int i, err;
+     int i;
      pkg_vec_t *available;
      pkg_t *pkg;
      char *pkg_name = NULL;
      char **pkg_fields = NULL;
      int n_fields = 0;
-     regex_t re;
      char *buff ; // = (char *)malloc(1);
 
      if (argc > 0) {
 	  pkg_name = argv[0];
-	  err = ipkg_regcomp(&re, pkg_name);
-	  if (err) {
-	       return err;
-	  }
      }
      if (argc > 1) {
 	  pkg_fields = &argv[1];
@@ -586,7 +575,7 @@ static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char **argv, int in
 	  pkg_hash_fetch_available(&conf->pkg_hash, available);
      for (i=0; i < available->len; i++) {
 	  pkg = available->pkgs[i];
-	  if (pkg_name && xregexec(&re, pkg->name, 0, 0, 0)) {
+	  if (pkg_name && fnmatch(pkg_name, pkg->name, 0)) {
 	       continue;
 	  }
 #ifndef IPKG_LIB
@@ -626,8 +615,7 @@ static int ipkg_info_status_cmd(ipkg_conf_t *conf, int argc, char **argv, int in
 	  free(buff);
 #endif
      pkg_vec_free(available);
-     if (pkg_name)
-	  xregfree(&re);
+
      return 0;
 }
 
@@ -643,21 +631,15 @@ static int ipkg_status_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
 static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
-     int err;
      
+     int err;
      if (argc > 0) {
-          regex_t re;
 	  char *pkg_name = NULL;
 
 	  pkg_name = argv[0];
-	  err = ipkg_regcomp(&re, pkg_name);
-	  if (err) {
-	       return err;
-	  }
 
-	  err = ipkg_configure_packages (conf, &re);
+	  err = ipkg_configure_packages (conf, pkg_name);
      
-	  xregfree(&re);
      } else {
 	  err = ipkg_configure_packages (conf, NULL);
      }
@@ -707,13 +689,12 @@ static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
 static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
-     int i,err,a,done;
+     int i,a,done;
      pkg_t *pkg;
      pkg_t *pkg_to_remove;
      pkg_vec_t *available;
      char *pkg_name = NULL;
      global_conf = conf;
-     regex_t re;
      signal(SIGINT, sigint_handler);
 
 // ENH: Add the "no pkg removed" just in case.
@@ -725,16 +706,11 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
      if ( argc > 0 ) {
         pkg_hash_fetch_all_installed(&conf->pkg_hash, available);
         for (i=0; i < argc; i++) {
-           // pkg_name = argv[i];
            pkg_name = malloc(strlen(argv[i])+2);
            strcpy(pkg_name,argv[i]);
-           err = ipkg_regcomp(&re, pkg_name);
-           if (err) {
-               return err;
-           }
            for (a=0; a < available->len; a++) {
                pkg = available->pkgs[a];
-               if (pkg_name && xregexec(&re, pkg->name, 0, 0, 0)) {
+	       if (pkg_name && fnmatch(pkg_name, pkg->name, 0)) {
                   continue;
                }
                if (conf->restrict_to_default_dest) {
@@ -755,11 +731,7 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
                }
                ipkg_remove_pkg(conf, pkg_to_remove);
                done = 1;
-               // a=available->len;  // Let's go out from here. We should remove the next one.
-               // pkg_hash_fetch_all_installed(&conf->pkg_hash, available);  // This could not be needed, but just in case....
            }
-           if (pkg_name)
-              xregfree(&re);
            free (pkg_name);
         }
         pkg_vec_free(available);
@@ -892,6 +864,7 @@ static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char **argv)
      str_list_elt_t *iter;
      char *pkg_version;
      size_t buff_len = 8192;
+     size_t used_len;
      char *buff ;
 
      buff = (char *)malloc(buff_len);
@@ -923,9 +896,20 @@ static int ipkg_files_cmd(ipkg_conf_t *conf, int argc, char **argv)
      }
 #else
      if (buff) {
-	  snprintf(buff, buff_len, "Package %s (%s) is installed on %s and has the following files:\n",
-		   pkg->name, pkg_version, pkg->dest->name);
+     try_again:
+	  used_len = snprintf(buff, buff_len, "Package %s (%s) is installed on %s and has the following files:\n",
+			      pkg->name, pkg_version, pkg->dest->name) + 1;
+	  if (used_len > buff_len) {
+	       buff_len *= 2;
+	       buff = realloc (buff, buff_len);
+	       goto try_again;
+	  }
 	  for (iter = installed_files->head; iter; iter = iter->next) {
+	       used_len += strlen (iter->data) + 1;
+	       while (buff_len <= used_len) {
+		    buff_len *= 2;
+		    buff = realloc (buff, buff_len);
+	       }
 	       strncat(buff, iter->data, buff_len);
 	       strncat(buff, "\n", buff_len);
 	  } 
@@ -960,15 +944,13 @@ static int ipkg_depends_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	       pkg_hash_fetch_all_installed(&conf->pkg_hash, available_pkgs);
 	  for (i = 0; i < argc; i++) {
 	       const char *target = argv[i];
-	       regex_t re;
 	       int j;
 
 	       ipkg_message(conf, IPKG_ERROR, "target=%s\n", target);
 
-	       ipkg_regcomp(&re, target);
 	       for (j = 0; j < available_pkgs->len; j++) {
 		    pkg_t *pkg = available_pkgs->pkgs[j];
-		    if (xregexec(&re, pkg->name, 0, 0, 0) == 0) {
+		    if (fnmatch(target, pkg->name, 0) == 0) {
 			 int k;
 			 int count = pkg->depends_count + pkg->pre_depends_count;
 			 ipkg_message(conf, IPKG_ERROR, "What %s (arch=%s) %s\n",
@@ -1168,10 +1150,8 @@ static int ipkg_what_provides_replaces_cmd(ipkg_conf_t *conf, enum what_field_ty
 	       pkg_hash_fetch_all_installed(&conf->pkg_hash, available_pkgs);
 	  for (i = 0; i < argc; i++) {
 	       const char *target = argv[i];
-	       regex_t re;
 	       int j;
 
-	       ipkg_regcomp(&re, target);
 	       ipkg_message(conf, IPKG_ERROR, "What %s %s\n",
 			    rel_str, target);
 	       for (j = 0; j < available_pkgs->len; j++) {
@@ -1183,7 +1163,7 @@ static int ipkg_what_provides_replaces_cmd(ipkg_conf_t *conf, enum what_field_ty
 			      ((what_field_type == WHATPROVIDES) 
 			       ? pkg->provides[k]
 			       : pkg->replaces[k]);
-			 if (xregexec(&re, apkg->name, 0, 0, 0) == 0) {
+			 if (fnmatch(target, apkg->name, 0) == 0) {
 			      ipkg_message(conf, IPKG_ERROR, "    %s", pkg->name);
 			      if (strcmp(target, apkg->name) != 0)
 				   ipkg_message(conf, IPKG_ERROR, "\t%s %s\n", rel_str, apkg->name);
@@ -1209,8 +1189,7 @@ static int ipkg_whatreplaces_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
 static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
-     int i, err;
-     regex_t re;
+     int i;
 
      pkg_vec_t *installed;
      pkg_t *pkg;
@@ -1222,11 +1201,6 @@ static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	  return EINVAL;
      }
  
-     err = ipkg_regcomp(&re, argv[0]);
-     if (err) {
-	  return err;
-     }
-
      installed = pkg_vec_alloc();
      pkg_hash_fetch_all_installed(&conf->pkg_hash, installed);
 
@@ -1237,7 +1211,7 @@ static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
 	  for (iter = installed_files->head; iter; iter = iter->next) {
 	       installed_file = iter->data;
-	       if (xregexec(&re, installed_file, 0, 0, 0) == 0) {
+	       if (fnmatch(argv[0], installed_file, 0)==0)  {
 #ifndef IPKG_LIB
 		    printf("%s: %s\n", pkg->name, installed_file);
 #else
@@ -1257,8 +1231,6 @@ static int ipkg_search_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	pkg_vec_free to avoid a memory leak. */
      pkg_vec_free(installed);
 
-     regfree(&re);
-
      return 0;
 }
 
@@ -1297,3 +1269,4 @@ static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char **argv)
      return 0;
 }
 
+
